import click


def bar_progress(current, total, width=80):
    """
    Create a progress bar.

    Args:
        current (int): The current progress.
        total (int): The total progress.
        width (int): The width of the progress bar.

    Returns:
        str: The progress bar.
    """
    if total == 0:
        return "[{}{}]".format("#" * (width - 2), " " * 0)
    avail_dots = width - 2
    shaded_dots = int(current / total * avail_dots)
    return f"[{'#' * shaded_dots}{' ' * (avail_dots - shaded_dots)}]"


def parse_headers(header_list, verbose=False):
    """
    Parse a list of headers into a dictionary.

    Args:
        header_list (list): A list of headers.
        verbose (bool): Enable verbose mode.

    Returns:
        dict: A dictionary of headers.
    """
    headers = {}
    for header in header_list:
        key, value = header.split(":", 1)
        headers[key.strip()] = value.strip()
    if verbose:
        click.echo(click.style(f"Extra Headers Supplied: {headers}", fg="blue"))
    return headers



#!/usr/bin/env python3
"""
Download utility as an easy way to get files from the net
"""
import click

from .download import download_file
from .utils import parse_headers


@click.command()
@click.argument("url")
@click.option(
    "-o",
    "--output_file",
    default=None,
    help="Optional output file name",
)
@click.option(
    "--header",
    multiple=True,
    help="Custom headers to include in the request, e.g. --header 'Authorization: Bearer token', --header 'Content-Type: application/json', --header 'User-Agent: Mozilla/5.0', etc.",
)
@click.option(
    "--verbose",
    is_flag=True,
    help="Enable verbose mode to output detailed information about the download process.",
)
@click.version_option(prog_name="PyWget")
def main(url, output_file, header, verbose):
    """
    Download utility to fetch a file from the internet.

    Args:
        url (str): The URL of the file to download.
        output_file (str): The name of the file to save the downloaded file as.
        header (list): Optional HTTP headers to include in the request.
        verbose (bool): Enable verbose mode.

    Returns:
        None
    """
    if verbose:
        click.echo(click.style(f"Downloading {url} to {output_file}", fg="yellow"))
    headers = parse_headers(header, verbose=verbose)
    filename = download_file(url, output_file, headers, verbose=verbose)
    click.echo(click.style(f"\nSaved  {url} as {filename}", fg="green"))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
Download utility as an easy way to get files from the net

This script is not option compatible with Unix wget utility,
to make the command line interface intuitive for new users.

Public domain by ktechhub <info@ktechhub.com>
Also available under the terms of MIT license
Copyright (c) 2024 ktechhub
"""



#!/usr/bin/env python3
"""
Download utility as an easy way to get files from the net
"""
import click
import sys
import os
import shutil
import tempfile
import requests
from urllib.parse import urlparse

from .utils import bar_progress


def filename_from_url(url):
    """
    Get the filename from a URL.

    Args:
        url (str): The URL to get the filename from.

    Returns:
        str: The filename from the URL.
    """
    return os.path.basename(urlparse(url).path) or "download.wget"


def download_file(url, out=None, headers={}, verbose=False):
    """
    Download a file from the internet.

    Args:
        url (str): The URL of the file to download.
        out (str): The name of the file to save the downloaded file as.
        headers (dict): Optional HTTP headers to include in the request.

    Returns:
        str: The name of the file the downloaded file was saved as.
    """
    out = out or filename_from_url(url)
    tmpfile = tempfile.mktemp(dir=".", prefix=out, suffix=".tmp")

    with requests.get(url, headers=headers, stream=True) as r:
        r.raise_for_status()
        total_size = int(r.headers.get("content-length", 0))
        current_size = 0

        with open(tmpfile, "wb") as f:
            for chunk in r.iter_content(chunk_size=8192):
                if chunk:
                    f.write(chunk)
                    current_size += len(chunk)
                    progress_bar = bar_progress(current_size, total_size)
                    sys.stdout.write(
                        f"\r{progress_bar} {current_size}/{total_size if total_size else 'unknown'} bytes"
                    )
                    if verbose:
                        click.echo(
                            click.style(
                                f"\nDownloaded {current_size}/{total_size if total_size else 'unknown'} bytes",
                                fg="blue",
                            )
                        )
                    sys.stdout.flush()

    sys.stdout.write("\n")

    if os.path.exists(out):
        base, ext = os.path.splitext(out)
        count = 1
        while os.path.exists(out):
            out = f"{base} ({count}){ext}"
            count += 1

    shutil.move(tmpfile, out)
    if verbose:
        click.echo(click.style(f"Downloaded {url} to {out}", fg="green"))
    return out



"""
A setuptools based setup module.
See:
https://www.ktechhub.com/tutorials/how-to-package-a-python-code-and-upload-to-pypi
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

import io
from os import path, getenv
from setuptools import setup, find_packages

VERSION = getenv("VERSION", "0.0.3")  # package version
if "v" in VERSION:
    VERSION = VERSION[1:]

here = path.abspath(path.dirname(__file__))
# Get the long description from the README file
with io.open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = "\n" + f.read()

REQUIRES_PYTHON = ">=3.6"

setup(
    name="pywgett",
    version=VERSION,
    packages=find_packages(exclude=["tests", "docs", ".github"]),
    install_requires=["click", "requests"],
    entry_points={
        "console_scripts": [
            "pywget = pywgett.cli:main",
        ],
    },
    test_suite="tests",
    python_requires=REQUIRES_PYTHON,
    author="Ktechhub",
    author_email="mm@ktechhub.com",
    description="Download utility as an easy way to get files from the net",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ktechhub/pywgett",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.6",
    ],
    keywords="download utility wget pywgett pywget pywgett.py pywget.py",
    setup_requires=["wheel"],
    include_package_data=True,
)
